# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# NOTE: This file is auto generated by the elixir code generator program.
# Do not edit this file manually.

defmodule GoogleApi.Compute.V1.Model.BackendServiceConnectionTrackingPolicy do
  @moduledoc """
  Connection Tracking configuration for this BackendService.

  ## Attributes

  *   `connectionPersistenceOnUnhealthyBackends` (*type:* `String.t`, *default:* `nil`) - Specifies connection persistence when backends are unhealthy. The default value is DEFAULT_FOR_PROTOCOL. If set to DEFAULT_FOR_PROTOCOL, the existing connections persist on unhealthy backends only for connection-oriented protocols (TCP and SCTP) and only if the Tracking Mode is PER_CONNECTION (default tracking mode) or the Session Affinity is configured for 5-tuple. They do not persist for UDP. If set to NEVER_PERSIST, after a backend becomes unhealthy, the existing connections on the unhealthy backend are never persisted on the unhealthy backend. They are always diverted to newly selected healthy backends (unless all backends are unhealthy). If set to ALWAYS_PERSIST, existing connections always persist on unhealthy backends regardless of protocol and session affinity. It is generally not recommended to use this mode overriding the default. For more details, see [Connection Persistence for Network Load Balancing](https://cloud.google.com/load-balancing/docs/network/networklb-backend-service#connection-persistence) and [Connection Persistence for Internal TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/internal#connection-persistence).
  *   `enableStrongAffinity` (*type:* `boolean()`, *default:* `nil`) - Enable Strong Session Affinity for Network Load Balancing. This option is not available publicly.
  *   `idleTimeoutSec` (*type:* `integer()`, *default:* `nil`) - Specifies how long to keep a Connection Tracking entry while there is no matching traffic (in seconds). For Internal TCP/UDP Load Balancing: - The minimum (default) is 10 minutes and the maximum is 16 hours. - It can be set only if Connection Tracking is less than 5-tuple (i.e. Session Affinity is CLIENT_IP_NO_DESTINATION, CLIENT_IP or CLIENT_IP_PROTO, and Tracking Mode is PER_SESSION). For Network Load Balancer the default is 60 seconds. This option is not available publicly.
  *   `trackingMode` (*type:* `String.t`, *default:* `nil`) - Specifies the key used for connection tracking. There are two options: - PER_CONNECTION: This is the default mode. The Connection Tracking is performed as per the Connection Key (default Hash Method) for the specific protocol. - PER_SESSION: The Connection Tracking is performed as per the configured Session Affinity. It matches the configured Session Affinity. For more details, see [Tracking Mode for Network Load Balancing](https://cloud.google.com/load-balancing/docs/network/networklb-backend-service#tracking-mode) and [Tracking Mode for Internal TCP/UDP Load Balancing](https://cloud.google.com/load-balancing/docs/internal#tracking-mode).
  """

  use GoogleApi.Gax.ModelBase

  @type t :: %__MODULE__{
          :connectionPersistenceOnUnhealthyBackends => String.t() | nil,
          :enableStrongAffinity => boolean() | nil,
          :idleTimeoutSec => integer() | nil,
          :trackingMode => String.t() | nil
        }

  field(:connectionPersistenceOnUnhealthyBackends)
  field(:enableStrongAffinity)
  field(:idleTimeoutSec)
  field(:trackingMode)
end

defimpl Poison.Decoder, for: GoogleApi.Compute.V1.Model.BackendServiceConnectionTrackingPolicy do
  def decode(value, options) do
    GoogleApi.Compute.V1.Model.BackendServiceConnectionTrackingPolicy.decode(value, options)
  end
end

defimpl Poison.Encoder, for: GoogleApi.Compute.V1.Model.BackendServiceConnectionTrackingPolicy do
  def encode(value, options) do
    GoogleApi.Gax.ModelBase.encode(value, options)
  end
end
